--
-- Color management
--
package Video.Colors with Pure is

	Color_Component_Bits : constant := 8;
	type Color_Component is private;
	-- Universal definition for color component (like R, G, B channels or Alpha)
	-- defined to cover the best color resolution required (e. g. 8 bit in true color systems)

	subtype Color_Component_8_Bit is Integer range 0 .. 2**8 - 1;
	-- Integer representation of a color component in range 0 .. 255;

	function From_8_Bit (Value : Color_Component_8_Bit) return Color_Component with Inline;
	-- Convert from integer reptesentation

	function To_8_Bit (Value : Color_Component) return Color_Component_8_Bit with Inline;
	-- Comvert to integer representation

	type Color is record
		R : Color_Component;
		G : Color_Component;
		B : Color_Component;
		A : Color_Component;
	end record;

	function RGB_8 (
		R, G, B : Color_Component_8_Bit;
		A       : Color_Component_8_Bit := Color_Component_8_Bit'Last)
		return Color with Inline;
	-- Define color by R, G, B and alpha 8 bit components

private
	type Color_Component is range 0 .. 2**Color_Component_Bits - 1;

end Video.Colors;